GBLUP_GWAS_fixed <- function(geno, GID,  map, pheno, method = "QTL",
                             window = 10, nMksel = 10, threshold = 3,
                             verbose = FALSE, do_plot = FALSE){
  
  # format data
  colnames(map) <- c("mk.names","chr", "pos.cM")
  
  # a) GWAS ----
  K <- A.mat(geno)
  g_data <- data.frame(map, t(geno), check.names = FALSE)
  y_pheno <- data.frame(gid = rownames(pheno), y = pheno[, 1])
  B_GWAS <- rrBLUP::GWAS(pheno = y_pheno, geno = g_data, K = K, plot = do_plot)
  
  
  # b) marker selection ----
  if(method == "QTL"){
    Q_prof <- data.frame(mk.names = map[, 1], chr = map[, 2],
                         pos.ind = NA, pos.cM = map[, 3],
                         log10pval = B_GWAS$y)
    class(Q_prof) <- c("QTLprof", "data.frame")
    Qsel <- QTL_select(Qprof = Q_prof, threshold = threshold,
                       window = window, verbose = FALSE)
    if(!is.null(Qsel)){
      Q_sel_nm <- Qsel$mk.names
      if(verbose){print(paste("Selected positions:", paste(Q_sel_nm, collapse = ", ")))}
    } else {Q_sel_nm <- NULL}
    
  } else if (method == "RiceLipka"){
    B_GWAS <- B_GWAS[order(B_GWAS$y, decreasing = TRUE), ]
    Q_sel_nm <- B_GWAS[, 1][1:nMksel]
    if(verbose){print(paste("Selected positions:", paste(Q_sel_nm, collapse = ", ")))}
  }
  
  # c) geno matrix cleaning (window = 0 if method = "RiceLipka") ----
  if(!is.null(Q_sel_nm)){
    
    if(method == "RiceLipka"){
      # win_clean = 0
      geno_clean <- geno[, -which(map[, 1] %in% Q_sel_nm)]
    } else if (method == "QTL"){
      
      test.cof <- function(x, map, window) {
        
        t1 <- map$chr == as.numeric(x[1])
        t2 <- abs(map$pos.cM - as.numeric(x[2])) < window
        !(t1 & t2)
        
      }
      
      mk.part <- apply(X = Qsel[, c(2, 4)], MARGIN = 1,
                       FUN = test.cof,
                       map = map, window = window)
      sel_mk <- apply(mk.part, 1, function(x) all(x))
      
      geno_clean <- geno[, sel_mk]
      
    }
    
  } else{
    geno_clean <- geno
  }
  
  # d) GP ----
  
  # calculate kinship
  K <- A.mat(X = geno_clean)
  d <- data.frame(GID = GID, y = pheno)
  colnames(d)[2] <- "y"
  
  if(!is.null(Q_sel_nm)){
    # add the fixed marker terms
    d <- data.frame(d, geno[, which(map[, 1] %in% Q_sel_nm)])
    colnames(d)[3:ncol(d)] <- Q_sel_nm
    
    # m <- kin.blup(data = d, geno = "GID", pheno = "y",
    #               K = K, GAUSS = FALSE, fixed = Q_sel_nm)
    
    f_formula <- paste0("y~1+", paste0(Q_sel_nm, collapse = "+"))
    m <- mmer(fixed = as.formula(f_formula),
              random= ~vsr(GID,Gu=K),
              rcov= ~ units,
              data=d, verbose = FALSE, dateWarning = FALSE)
    
  } else {
    # m <- kin.blup(data = d, geno = "GID", pheno = "y",
    #               K = K, GAUSS = FALSE)
    
    m <- tryCatch(mmer(fixed = y~1,
                       random= ~vsr(GID,Gu=K),
                       rcov= ~ units,
                       data=d, verbose = FALSE, dateWarning = FALSE),
                  error = function(x) NULL)
    
    
  }
  
  # return GP output
  return(list(m = m, d = d, Q_sel = Q_sel_nm))
  
}